#!/usr/bin/env lua 
-- See the "conversions" table below to define 
-- which variables you want to convert, and how.
--
-- This converts pre-version 3.0 presets to version 3.0+ presets, adding
-- metadata.  The algo is set to ngfm-5b6b, as it is intended to take
-- Next Gen "alpha" presets as input.

local BASE_Fs = 48000
local BASE_Ts = 1/BASE_Fs
local BASE_Ts_ms = 1000/BASE_Fs

local log10_e = math.log10( math.exp(1) )
local twenty_log10_e = 20 * log10_e

local function fc2coef_1( fc, Ts )
-- calculate the coefficient needed in a first order "time constant style"
-- filter (i.e. without a zero at Nyquist) in order to make the -3dB
-- frequency equal to fc for a sample period of Ts
    local w = 2 * math.pi * fc * Ts
    local c = math.cos( w )
    return 2 - c - math.sqrt( (c - 4)*c + 3  )
end

local function tc2coef_1( tc, Ts )
-- calculate the coefficient needed to implement the RC-equivalent
-- time-constant (under impulse-invariant transform) in a first order
-- filter for a sample period of Ts.  Note that tc must be in the
-- same units as Ts.
-- The exact equation is a = 1 - exp( -(Ts/tc) ).  The expression below
-- is an approximation which is very good for tc of more than a few samples.
   return Ts / tc
end

--+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
--
-- specific variables to convert and the associated conversion 
-- functions here. 
--
local conversions = {

    loagc_ratio = function(x) return 0.25 end,
    mlagc_ratio = function(x) return 0.25 end,
    mhagc_ratio = function(x) return 0.25 end, 
    hiagc_ratio = function(x) return 0.25 end,
    shagc_ratio = function(x) return 0.25 end,
    wbagc_gate_thresh = function(x) return -28 end,
    lf_lim_density = function(x) return 20.0 * math.log10( x ) end,
    hf_lim_density = function(x) return 20.0 * math.log10( x ) end,
}
--+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

local function print_line( name, val )
    if type( val ) == "number" then
        assert( io.write( string.format("\t%s = %g,\n", name, val) ) )
    else
        assert( io.write( string.format("\t%s = %q,\n", name, val) ) )
    end
end

local function convert( name, val )
    local f = conversions[ name ]
    if f then
        print_line( name, f( val ) )
    else
        print_line( name, val )
    end  
end

local function convertPreset( fname )
   -- the preset file calls "Preset"
   function Preset( t ) 
      table.foreach( t, convert )
   end
   local f = loadfile( fname )
   if not f then return false end
   -- Now actually execute the preset file
   io.write( "Preset_metadata {\n" )
   io.write( "\talgo = 'ngfm-5b6b',\n" )
   io.write( "\talgo_ver = 1,\n" )
   io.write( "\ttags = { ['ngalpha'] = true },\n" )
   io.write( "\tnotes = [==========[\n" )
   io.write( "]==========],\n" )
   io.write( "}\n\n" )
   io.write( "if not Preset_metadata_only then\n" )
   io.write( "Preset {\n" )
   f()
   io.write( "}\n" )
   io.write( "end" )
   return true
end

assert( convertPreset( arg[1] ) )
